package com.tzld.piaoquan.recommend.server.service.recall.strategy;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.filter.FilterResult;
import com.tzld.piaoquan.recommend.server.service.filter.VideoFilterService;
import com.tzld.piaoquan.recommend.server.service.recall.FilterParamFactory;
import com.tzld.piaoquan.recommend.server.service.recall.RecallParam;
import com.tzld.piaoquan.recommend.server.service.recall.RecallStrategy;
import com.tzld.piaoquan.recommend.server.util.JSONUtils;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.*;

/**
 * @author dyp
 */
@Slf4j
public abstract class AbstractVideoRecallStrategy implements RecallStrategy {
    @Autowired
    protected RedisTemplate<String, String> redisTemplate;

    @Autowired
    protected VideoFilterService filterService;

    @Override
    public List<Video> recall(RecallParam param) {


        if (param.getVideoId() == null) {
            return Collections.emptyList();
        }
        String recall_key = recallKey(param);
        // [[16687915, 0.5], [16528093, 0.5]]
        String value = redisTemplate.opsForValue().get(recall_key);
        log.info("recall recall_key={}, value={}", recall_key, JSONUtils.toJson(value));
        if (StringUtils.isBlank(value)) {
            return null;
        }
        List<Item> items = JSONObject.parseArray(value, Item.class);
        List<Video> results = new ArrayList<>();

        Map<Long, Double> videoScoreMap = new HashMap<>();
        for (int i = 0; i < items.size() && i < 20; i++) {
            Item item = items.get(i);
            videoScoreMap.put(NumberUtils.toLong(item.getVideoId(), 0),
                    NumberUtils.toDouble(item.getScore(), 0));
        }

        FilterResult filterResult = filterService.filter(FilterParamFactory.create(param, Lists.newArrayList(videoScoreMap.keySet())));

        if (filterResult != null && CollectionUtils.isNotEmpty(filterResult.getVideoIds())) {
            filterResult.getVideoIds().stream().forEach(vid -> {
                Video recallData = new Video();
                recallData.setVideoId(vid);
                recallData.setAbCode(param.getAbCode());
                recallData.setRovScore(videoScoreMap.get(vid));
                recallData.setPushFrom(pushFrom());
                recallData.setFlowPool("");
                results.add(recallData);
            });
        }

        return results;
    }

    @Data
    class Item {
        private String videoId;
        private String score;
    }

    abstract String recallKey(RecallParam param);
}
